import { BigintIsh, Percent, Price, CurrencyAmount, Token, Fraction } from '@uniswap/sdk-core';
import JSBI from 'jsbi';
import { Pool, TransactionOverrides } from './pool';
import { ethers } from 'ethers';
import { IncreaseOptions, MintOptions, RemoveLiquidityOptions } from '../nonfungiblePositionManager';
interface PositionConstructorArgs {
    pool: Pool;
    tickLower: number;
    tickUpper: number;
    liquidity: BigintIsh;
    positionId?: BigintIsh;
    tokensOwed0?: BigintIsh;
    tokensOwed1?: BigintIsh;
}
/**
 * Represents a position on a Uniswap V3 Pool
 */
export declare class Position {
    readonly pool: Pool;
    readonly tickLower: number;
    readonly tickUpper: number;
    get liquidity(): JSBI;
    readonly _liquidity: bigint;
    readonly positionId?: bigint;
    readonly tokensOwed0?: bigint;
    readonly tokensOwed1?: bigint;
    private _token0Amount;
    private _token1Amount;
    private _mintAmounts;
    /**
     * Initializes the position from a given position id using on-chain data.
     *
     * @param provider The provider to use to fetch data.
     * @param positionId The position id to fetch.
     * @returns Instance of Position.
     */
    static fetchWithPositionId({ provider, positionId, }: {
        provider: ethers.providers.Provider;
        positionId: BigintIsh;
    }): Promise<Position>;
    /**
     * Returns the number of positions the given address owns. Using this, position identifiers can be fetched.
     * A position can always be fetched with the combination of (address, index), so for an address with 3 positions,
     * there are positions on index 0, 1 and 2.
     *
     * Use `getPositionForAddressAndIndex` to fetch individual positions in a paginated manner.
     *
     * @param provider The provider to use for fetching the position count.
     * @param address The address to fetch position count for.
     * @returns The number of positions of the given address.
     */
    static getPositionCount({ provider, address, }: {
        provider: ethers.providers.Provider;
        address: string;
    }): Promise<bigint>;
    /**
     * Returns the position of the given address at the given index.
     * You need to know the number of positions an address has first. You can use
     * `getPositionCount` for that.
     *
     * @param provider The provider to use to fetch the position.
     * @param address The address to fetch a position for.
     * @param index The index of the position for the given address to fetch.
     * @returns The initialized position.
     */
    static getPositionForAddressAndIndex({ provider, address, index, }: {
        provider: ethers.providers.Provider;
        address: string;
        index: BigintIsh;
    }): Promise<Position>;
    /**
     * WARNING: This is a potentially heavy call that takes up lots of RPC resources and can take many seconds
     * to resolve. This is the case if the address given has many positions (whether closed or open) on the network.
     * (e.g. thousands of positions are a real problem, but even 500 will take at least a few seconds).
     *
     * To create a paginated version of this, call `getPositionCount` and then `getPositionForAddressAndIndex` as many times
     * as you need to render your current page.
     *
     * Fetches all positions of the given address.
     *
     * @param provider The provider to use to fetch positions for.
     * @param address The address to fetch positions for.
     * @returns The initialized positions as an array.
     */
    static getAllPositionsForAddress(provider: ethers.providers.Provider, address: string): Promise<Position[]>;
    /**
     * Constructs a position for a given pool with the given liquidity
     * @param pool For which pool the liquidity is assigned
     * @param liquidity The amount of liquidity that is in the position
     * @param tickLower The lower tick of the position
     * @param tickUpper The upper tick of the position
     * @param positionId (optional) The positionId of the existing position on-chain.
     * @param tokensOwed0 (optional) The fees owed to the position in token0.
     * @param tokensOwed1 (optional) The fees owed to the position in token1.
     */
    constructor({ pool, liquidity, tickLower, tickUpper, positionId, tokensOwed0, tokensOwed1, }: PositionConstructorArgs);
    /**
     * Creates the position on-chain and mints the NFT. Can only be called if `positionId` is null, e.g.
     * the position wasn't created yet.
     *
     * You can provide mint options in `options`, including recipient of the NFT.
     *
     * @param signer The signer to use to sign and send the transaction.
     * @param provider The provider to use to propagate the transaction.
     * @param options The mint options.
     * @param transactionOverrides If you want to customize gas limit, gas price, etc.
     *
     * @returns The transaction response including hash. You need to wait for transaction inclusion yourself.
     */
    mint({ signer, provider, options, transactionOverrides, }: {
        signer: ethers.Signer;
        provider: ethers.providers.Provider;
        options: MintOptions;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    /**
     * Increase the position by the given percentage of the current position size and create a transaction for the change.
     *
     * e.g.: 10% => Add 10% of current amount0 and amount1 to the position, so the new position will be 10% higher than before.
     *
     * This function requires gas.
     *
     * @param signer The signer to use to sign and send the transaction.
     * @param provider The provider to use to propagate the transaction.
     * @param percentage The percentage of the current amounts to increase by.
     * @param options The increase options.
     * @param transactionOverrides If you want to customize gas limit, gas price, etc.
     *
     * @returns The transaction response including hash. You need to wait for transaction inclusion yourself.
     */
    increasePositionByPercentageOnChain({ signer, provider, percentage, options, transactionOverrides, }: {
        signer: ethers.Signer;
        provider: ethers.providers.Provider;
        percentage: Fraction;
        options: IncreaseOptions;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    /**
     * Decrease the position by the given percentage of the current position size and create a transaction for the change.
     *
     * e.g.: 10% => Remove 10% of current amount0 and amount1 from the position, so the new position will be 10% lower than before.
     *
     * This function requires gas.
     *
     * @param signer The signer to use to sign and send the transaction.
     * @param provider The provider to use to propagate the transaction.
     * @param percentage The percentage of the current amounts to decrease by.
     * @param options The remove liquidity options.
     * @param transactionOverrides If you want to customize gas limit, gas price, etc.
     *
     * @returns The transaction response including hash. You need to wait for transaction inclusion yourself.
     */
    decreasePositionByPercentageOnChain({ signer, provider, percentage, options, transactionOverrides, }: {
        signer: ethers.Signer;
        provider: ethers.providers.Provider;
        percentage: Fraction;
        options: Omit<RemoveLiquidityOptions, 'liquidityPercentage' | 'collectOptions'>;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    collectFeesOnChain({ signer, provider, percentage, transactionOverrides, }: {
        signer: ethers.Signer;
        provider: ethers.providers.Provider;
        percentage?: Fraction;
        transactionOverrides?: TransactionOverrides;
    }): Promise<ethers.providers.TransactionResponse>;
    /**
     * Returns the price of token0 at the lower tick
     */
    get token0PriceLower(): Price<Token, Token>;
    /**
     * Returns the price of token0 at the upper tick
     */
    get token0PriceUpper(): Price<Token, Token>;
    /**
     * Returns the amount of token0 that this position's liquidity could be burned for at the current pool price
     */
    get amount0(): CurrencyAmount<Token>;
    /**
     * Returns the amount of token1 that this position's liquidity could be burned for at the current pool price
     */
    get amount1(): CurrencyAmount<Token>;
    /**
     * Returns the lower and upper sqrt ratios if the price 'slips' up to slippage tolerance percentage
     * @param slippageTolerance The amount by which the price can 'slip' before the transaction will revert
     * @returns The sqrt ratios after slippage
     */
    private ratiosAfterSlippage;
    /**
     * Returns the minimum amounts that must be sent in order to safely mint the amount of liquidity held by the position
     * with the given slippage tolerance
     * @param slippageTolerance Tolerance of unfavorable slippage from the current price
     * @returns The amounts, with slippage
     */
    mintAmountsWithSlippageBigInt(slippageTolerance: Percent): Readonly<{
        amount0: bigint;
        amount1: bigint;
    }>;
    /**
     * Returns the minimum amounts that must be sent in order to safely mint the amount of liquidity held by the position
     * with the given slippage tolerance
     * @param slippageTolerance Tolerance of unfavorable slippage from the current price
     * @returns The amounts, with slippage
     */
    mintAmountsWithSlippage(slippageTolerance: Percent): Readonly<{
        amount0: JSBI;
        amount1: JSBI;
    }>;
    /**
     * Returns the minimum amounts that should be requested in order to safely burn the amount of liquidity held by the
     * position with the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the current price
     * @returns The amounts, with slippage
     */
    burnAmountsWithSlippageBigInt(slippageTolerance: Percent): Readonly<{
        amount0: bigint;
        amount1: bigint;
    }>;
    /**
     * Returns the minimum amounts that should be requested in order to safely burn the amount of liquidity held by the
     * position with the given slippage tolerance
     * @param slippageTolerance tolerance of unfavorable slippage from the current price
     * @returns The amounts, with slippage
     */
    burnAmountsWithSlippage(slippageTolerance: Percent): Readonly<{
        amount0: JSBI;
        amount1: JSBI;
    }>;
    /**
     * Returns the minimum amounts that must be sent in order to mint the amount of liquidity held by the position at
     * the current price for the pool
     */
    get mintAmountsBigInt(): Readonly<{
        amount0: bigint;
        amount1: bigint;
    }>;
    /**
     * Returns the minimum amounts that must be sent in order to mint the amount of liquidity held by the position at
     * the current price for the pool
     */
    get mintAmounts(): Readonly<{
        amount0: JSBI;
        amount1: JSBI;
    }>;
    /**
     * Computes the maximum amount of liquidity received for a given amount of token0, token1,
     * and the prices at the tick boundaries.
     * @param pool The pool for which the position should be created
     * @param tickLower The lower tick of the position
     * @param tickUpper The upper tick of the position
     * @param amount0 token0 amount
     * @param amount1 token1 amount
     * @param useFullPrecision If false, liquidity will be maximized according to what the router can calculate,
     * not what core can theoretically support
     * @returns The amount of liquidity for the position
     */
    static fromAmounts({ pool, tickLower, tickUpper, amount0, amount1, useFullPrecision, }: {
        pool: Pool;
        tickLower: number;
        tickUpper: number;
        amount0: BigintIsh;
        amount1: BigintIsh;
        useFullPrecision: boolean;
    }): Position;
    /**
     * Computes a position with the maximum amount of liquidity received for a given amount of token0, assuming an unlimited amount of token1
     * @param pool The pool for which the position is created
     * @param tickLower The lower tick
     * @param tickUpper The upper tick
     * @param amount0 The desired amount of token0
     * @param useFullPrecision If true, liquidity will be maximized according to what the router can calculate,
     * not what core can theoretically support
     * @returns The position
     */
    static fromAmount0({ pool, tickLower, tickUpper, amount0, useFullPrecision, }: {
        pool: Pool;
        tickLower: number;
        tickUpper: number;
        amount0: BigintIsh;
        useFullPrecision: boolean;
    }): Position;
    /**
     * Computes a position with the maximum amount of liquidity received for a given amount of token1, assuming an unlimited amount of token0
     * @param pool The pool for which the position is created
     * @param tickLower The lower tick
     * @param tickUpper The upper tick
     * @param amount1 The desired amount of token1
     * @returns The position
     */
    static fromAmount1({ pool, tickLower, tickUpper, amount1, }: {
        pool: Pool;
        tickLower: number;
        tickUpper: number;
        amount1: BigintIsh;
    }): Position;
}
export {};
