import { CurrencyAmount, validateAndParseAddress, NONFUNGIBLE_POSITION_MANAGER_ADDRESSES, } from '@uniswap/sdk-core';
import invariant from 'tiny-invariant';
import { Position } from './entities/position';
import { toHex } from './utils/calldata';
import { Interface } from '@ethersproject/abi';
import INonfungiblePositionManager from '@uniswap/v3-periphery/artifacts/contracts/NonfungiblePositionManager.sol/NonfungiblePositionManager.json';
import { SelfPermit } from './selfPermit';
import { ADDRESS_ZERO } from './constants';
import { Pool } from './entities';
import { Multicall } from './multicall';
import { Payments } from './payments';
import { ethers } from 'ethers';
import { bigIntFromBigintIsh } from './utils/bigintIsh';
const MaxUint128 = `0x${(2n ** 128n - 1n).toString(16)}`;
// type guard
function isMint(options) {
    return Object.keys(options).some((k) => k === 'recipient');
}
export class NonfungiblePositionManager {
    /**
     * Cannot be constructed.
     */
    constructor() { }
    static encodeCreate(pool) {
        return NonfungiblePositionManager.INTERFACE.encodeFunctionData('createAndInitializePoolIfNecessary', [
            pool.token0.address,
            pool.token1.address,
            pool.fee,
            toHex(pool.sqrtRatioX96),
        ]);
    }
    /**
     * Create and if necessary initialize the pool.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param pool The pool to create and initialize.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async createOnChain(_signer, provider, pool, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const contract = new ethers.Contract(NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId], INonfungiblePositionManager.abi, signer);
        const response = contract.createAndInitializePoolIfNecessary(pool.token0.address, pool.token1.address, pool.fee, toHex(pool.sqrtRatioX96), Pool.ethersTransactionOverrides(transactionOverrides));
        return response;
    }
    static createCallParameters(pool) {
        return {
            calldata: this.encodeCreate(pool),
            value: toHex(0),
        };
    }
    /**
     * Creates a new position and mints the NFT of the position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to add liquidity to.
     * @param options Mint liquidity Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async createPositionOnChain(_signer, provider, position, options, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const data = this.addCallParameters(position, options);
        const response = signer.sendTransaction({
            to: NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId],
            data: data.calldata,
            value: data.value,
            gasPrice: transactionOverrides?.gasPrice?.toString(10),
            gasLimit: transactionOverrides?.gasLimit?.toString(10),
            nonce: transactionOverrides?.nonce?.toString(10),
        });
        return response;
    }
    /**
     * Add liquidity to position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to add liquidity to.
     * @param options Increase liquidity Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async increasePositionOnChain(_signer, provider, position, options, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const data = this.addCallParameters(position, options);
        const response = signer.sendTransaction({
            to: NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId],
            data: data.calldata,
            value: data.value,
            gasPrice: transactionOverrides?.gasPrice?.toString(10),
            gasLimit: transactionOverrides?.gasLimit?.toString(10),
            nonce: transactionOverrides?.nonce?.toString(10),
        });
        return response;
    }
    /**
     * Returns calldata for creating a position and minting the NFT.
     *
     * @param position The position to create.
     * @param options The mint options
     * @returns calldata and value to be used for signing and sending a transaction.
     */
    static createPositionCallParameters(position, options) {
        return this.addCallParameters(position, options);
    }
    /**
     * Returns calldata for increasing an existing position.
     *
     * @param position The position to create.
     * @param options The increase options
     * @returns calldata and value to be used for signing and sending a transaction.
     */
    static increasePositionCallParameters(position, options) {
        return this.addCallParameters(position, options);
    }
    static addCallParameters(position, options) {
        invariant(position._liquidity > 0n, 'ZERO_LIQUIDITY');
        const calldatas = [];
        // get amounts
        const { amount0: amount0Desired, amount1: amount1Desired } = position.mintAmountsBigInt;
        // adjust for slippage
        const minimumAmounts = position.mintAmountsWithSlippage(options.slippageTolerance);
        const amount0Min = toHex(minimumAmounts.amount0);
        const amount1Min = toHex(minimumAmounts.amount1);
        const deadline = toHex(options.deadline);
        // create pool if needed
        if (isMint(options) && options.createPool) {
            calldatas.push(this.encodeCreate(position.pool));
        }
        // permits if necessary
        if (options.token0Permit) {
            calldatas.push(SelfPermit.encodePermit(position.pool.token0, options.token0Permit));
        }
        if (options.token1Permit) {
            calldatas.push(SelfPermit.encodePermit(position.pool.token1, options.token1Permit));
        }
        // mint
        if (isMint(options)) {
            const recipient = validateAndParseAddress(options.recipient);
            calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('mint', [
                {
                    token0: position.pool.token0.address,
                    token1: position.pool.token1.address,
                    fee: position.pool.fee,
                    tickLower: position.tickLower,
                    tickUpper: position.tickUpper,
                    amount0Desired: toHex(amount0Desired),
                    amount1Desired: toHex(amount1Desired),
                    amount0Min,
                    amount1Min,
                    recipient,
                    deadline,
                },
            ]));
        }
        else {
            // increase
            calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('increaseLiquidity', [
                {
                    tokenId: toHex(options.tokenId),
                    amount0Desired: toHex(amount0Desired),
                    amount1Desired: toHex(amount1Desired),
                    amount0Min,
                    amount1Min,
                    deadline,
                },
            ]));
        }
        let value = toHex(0);
        if (options.useNative) {
            const wrapped = options.useNative.wrapped;
            invariant(position.pool.token0.equals(wrapped) || position.pool.token1.equals(wrapped), 'NO_WETH');
            const wrappedValue = position.pool.token0.equals(wrapped) ? amount0Desired : amount1Desired;
            // we only need to refund if we're actually sending ETH
            if (wrappedValue > 0n) {
                calldatas.push(Payments.encodeRefundETH());
            }
            value = toHex(wrappedValue);
        }
        return {
            calldata: Multicall.encodeMulticall(calldatas),
            value,
        };
    }
    static encodeCollect(options) {
        const calldatas = [];
        const tokenId = toHex(options.tokenId);
        const involvesETH = options.expectedCurrencyOwed0.currency.isNative || options.expectedCurrencyOwed1.currency.isNative;
        const recipient = validateAndParseAddress(options.recipient);
        // collect
        calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('collect', [
            {
                tokenId,
                recipient: involvesETH ? ADDRESS_ZERO : recipient,
                amount0Max: MaxUint128,
                amount1Max: MaxUint128,
            },
        ]));
        if (involvesETH) {
            const ethAmount = options.expectedCurrencyOwed0.currency.isNative
                ? options.expectedCurrencyOwed0.quotient
                : options.expectedCurrencyOwed1.quotient;
            const token = options.expectedCurrencyOwed0.currency.isNative
                ? options.expectedCurrencyOwed1.currency
                : options.expectedCurrencyOwed0.currency;
            const tokenAmount = options.expectedCurrencyOwed0.currency.isNative
                ? options.expectedCurrencyOwed1.quotient
                : options.expectedCurrencyOwed0.quotient;
            calldatas.push(Payments.encodeUnwrapWETH9(ethAmount, recipient));
            calldatas.push(Payments.encodeSweepToken(token, tokenAmount, recipient));
        }
        return calldatas;
    }
    /**
     * Collect fees on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param options Collect Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async collectOnChain(_signer, provider, options, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const data = this.collectCallParameters(options);
        const response = signer.sendTransaction({
            to: NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId],
            data: data.calldata,
            value: data.value,
            gasPrice: transactionOverrides?.gasPrice?.toString(10),
            gasLimit: transactionOverrides?.gasLimit?.toString(10),
            nonce: transactionOverrides?.nonce?.toString(10),
        });
        return response;
    }
    static collectCallParameters(options) {
        const calldatas = NonfungiblePositionManager.encodeCollect(options);
        return {
            calldata: Multicall.encodeMulticall(calldatas),
            value: toHex(0),
        };
    }
    /**
     * Remove a position on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param position The position to remove.
     * @param options Remove Options.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async removeOnChain(_signer, provider, position, options, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const data = this.removeCallParameters(position, options);
        const response = signer.sendTransaction({
            to: NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId],
            data: data.calldata,
            value: data.value,
            gasPrice: transactionOverrides?.gasPrice?.toString(10),
            gasLimit: transactionOverrides?.gasLimit?.toString(10),
            nonce: transactionOverrides?.nonce?.toString(10),
        });
        return response;
    }
    /**
     * Produces the calldata for completely or partially exiting a position
     * @param position The position to exit
     * @param options Additional information necessary for generating the calldata
     * @returns The call parameters
     */
    static removeCallParameters(position, options) {
        const calldatas = [];
        const deadline = toHex(options.deadline);
        const tokenId = toHex(options.tokenId);
        // construct a partial position with a percentage of liquidity
        const partialPosition = new Position({
            pool: position.pool,
            liquidity: options.liquidityPercentage.multiply(position.liquidity).quotient,
            tickLower: position.tickLower,
            tickUpper: position.tickUpper,
        });
        invariant(partialPosition._liquidity > 0n, 'ZERO_LIQUIDITY');
        // slippage-adjusted underlying amounts
        const { amount0: amount0Min, amount1: amount1Min } = partialPosition.burnAmountsWithSlippage(options.slippageTolerance);
        if (options.permit) {
            calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('permit', [
                validateAndParseAddress(options.permit.spender),
                tokenId,
                toHex(options.permit.deadline),
                options.permit.v,
                options.permit.r,
                options.permit.s,
            ]));
        }
        // remove liquidity
        calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('decreaseLiquidity', [
            {
                tokenId,
                liquidity: toHex(partialPosition.liquidity),
                amount0Min: toHex(amount0Min),
                amount1Min: toHex(amount1Min),
                deadline,
            },
        ]));
        const { expectedCurrencyOwed0, expectedCurrencyOwed1, ...rest } = options.collectOptions;
        calldatas.push(...NonfungiblePositionManager.encodeCollect({
            tokenId: toHex(options.tokenId),
            // add the underlying value to the expected currency already owed
            expectedCurrencyOwed0: expectedCurrencyOwed0.add(CurrencyAmount.fromRawAmount(expectedCurrencyOwed0.currency, amount0Min)),
            expectedCurrencyOwed1: expectedCurrencyOwed1.add(CurrencyAmount.fromRawAmount(expectedCurrencyOwed1.currency, amount1Min)),
            ...rest,
        }));
        if (options.liquidityPercentage.equalTo(1n)) {
            if (options.burnToken) {
                calldatas.push(NonfungiblePositionManager.INTERFACE.encodeFunctionData('burn', [tokenId]));
            }
        }
        else {
            invariant(options.burnToken !== true, 'CANNOT_BURN');
        }
        return {
            calldata: Multicall.encodeMulticall(calldatas),
            value: toHex(0),
        };
    }
    /**
     * Safe transfer from on-chain.
     *
     * This tx requires gas.
     *
     * @param _signer The signer to use to sign the transaction.
     * @param provider The provider to use to propagate the tx.
     * @param options SafeTransferOptions.
     * @param transactionOverrides If you want to set custom gas limit, nonce, etc. optional.
     * @returns The transaction response. You will still need to wait for inclusion.
     */
    static async safeTransferFromOnChain(_signer, provider, options, transactionOverrides) {
        let signer = _signer;
        try {
            signer = _signer.connect(provider);
        }
        catch { }
        const chainId = (await provider.getNetwork()).chainId;
        const data = this.safeTransferFromParameters(options);
        const response = signer.sendTransaction({
            to: NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId],
            data: data.calldata,
            value: data.value,
            gasPrice: transactionOverrides?.gasPrice?.toString(10),
            gasLimit: transactionOverrides?.gasLimit?.toString(10),
            nonce: transactionOverrides?.nonce?.toString(10),
        });
        return response;
    }
    static safeTransferFromParameters(options) {
        const recipient = validateAndParseAddress(options.recipient);
        const sender = validateAndParseAddress(options.sender);
        let calldata;
        if (options.data) {
            calldata = NonfungiblePositionManager.INTERFACE.encodeFunctionData('safeTransferFrom(address,address,uint256,bytes)', [sender, recipient, toHex(options.tokenId), options.data]);
        }
        else {
            calldata = NonfungiblePositionManager.INTERFACE.encodeFunctionData('safeTransferFrom(address,address,uint256)', [
                sender,
                recipient,
                toHex(options.tokenId),
            ]);
        }
        return {
            calldata: calldata,
            value: toHex(0),
        };
    }
    /**
     * Returns a URI describing a particular positionId.
     *
     * @param provider The provider to fetch from.
     * @param positionId The tokenId to fetch.
     *
     * @returns The positionId URI as a base64 string with URI data.
     */
    static async tokenURI(provider, positionId) {
        const chainId = (await provider.getNetwork()).chainId;
        const contract = new ethers.Contract(NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId], INonfungiblePositionManager.abi, provider);
        return await contract.tokenURI(ethers.BigNumber.from(bigIntFromBigintIsh(positionId).toString(10)));
    }
    /**
     * Returns the base URI of positions.
     *
     * @param provider The provider to fetch from.
     *
     * @returns The base URI.
     */
    static async baseURI(provider) {
        const chainId = (await provider.getNetwork()).chainId;
        const contract = new ethers.Contract(NONFUNGIBLE_POSITION_MANAGER_ADDRESSES[chainId], INonfungiblePositionManager.abi, provider);
        return await contract.baseURI();
    }
}
NonfungiblePositionManager.INTERFACE = new Interface(INonfungiblePositionManager.abi);
//# sourceMappingURL=nonfungiblePositionManager.js.map