import { Interface } from '@ethersproject/abi';
import { Provider } from '@ethersproject/abstract-provider';
import { BigintIsh, Currency, CurrencyAmount, TradeType, Token } from '@uniswap/sdk-core';
import { MethodParameters } from './utils';
import { Route } from './entities';
import { FeeAmount } from './constants';
/**
 * Optional arguments to send to the quoter.
 */
export interface QuoteOptions {
    /**
     * The optional price limit for the trade.
     */
    sqrtPriceLimitX96?: BigintIsh;
    /**
     * The optional quoter interface to use
     */
    useQuoterV2?: boolean;
}
/**
 * Represents the Uniswap V3 QuoterV1 and QuoterV2 contract.
 */
export declare abstract class SwapQuoter {
    static V1INTERFACE: Interface;
    static V2INTERFACE: Interface;
    /**
     * Fetches quotes for single hop exactInput swaps without the need to initialise a Route
     * @param amountIn The exact input amount
     * @param tokenOut The output token
     * @param poolFee The fee of the Pool used
     * @param provider a provider to make onchain calls
     * @returns The quoted output amount of the swap
     */
    static quoteExactInputSingle<TInput extends Token, TOutput extends Token>({ amountIn, tokenOut, poolFee, provider }: {
        amountIn: CurrencyAmount<TInput>;
        tokenOut: TOutput;
        poolFee: FeeAmount;
        provider: Provider;
    }): Promise<CurrencyAmount<TOutput>>;
    /**
     * Fetches quotes for single hop exactOutput swaps without the need to initialise a Route
     * @param tokenIn The input Token of the Swap
     * @param amountOut The exact output amount of the swap
     * @param poolFee to identify the Pool used
     * @param provider to make onchain calls
     * @returns The inputamount needed for the swap
     */
    static quoteExactOutputSingle<TInput extends Token, TOutput extends Token>({ tokenIn, amountOut, poolFee, provider }: {
        tokenIn: TInput;
        amountOut: CurrencyAmount<TOutput>;
        poolFee: FeeAmount;
        provider: Provider;
    }): Promise<CurrencyAmount<TInput>>;
    /**
     * Produces the on-chain method name of the appropriate function within QuoterV2,
     * and the relevant hex encoded parameters.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param route The swap route, a list of pools through which a swap can occur
     * @param amount The amount of the quote, either an amount in, or an amount out
     * @param tradeType The trade type, either exact input or exact output
     * @param options The optional params including price limit and Quoter contract switch
     * @returns The formatted calldata
     */
    static quoteCallParameters<TInput extends Currency, TOutput extends Currency>(route: Route<TInput, TOutput>, amount: CurrencyAmount<TInput | TOutput>, tradeType: TradeType, options?: QuoteOptions): MethodParameters;
    /**
     * Utility function to directly return a quote without the need to handle calldata from the user.
     * Uses quoteCallParameters internally. Always uses QuoterV2.
     * @template TInput The input token, either Ether or an ERC-20
     * @template TOutput The output token, either Ether or an ERC-20
     * @param route The swap route, a list of pools through which a swap can occur
     * @param amount The amount of the quote, either an amount in, or an amount out
     * @param tradeType The trade type, either exact input or exact output
     * @returns The decoded result of the Quoter call
     */
    static callQuoter<TInput extends Currency, TOutput extends Currency>({ route, amount, tradeType, provider }: {
        route: Route<TInput, TOutput>;
        amount: CurrencyAmount<TInput | TOutput>;
        tradeType: TradeType;
        provider: Provider;
    }): Promise<CurrencyAmount<TInput | TOutput>>;
}
