import { Contract } from '@ethersproject/contracts';
import { ERC20_ABI } from '../constants';
import { BigNumber } from 'ethers';
/**
 * Approves an address to transfer ERC20 Tokens on behalf of a Signer
 * @param contractAddress The address that is approved to transfer Tokens on behalf of a Signer
 * @param tokenAddress The address of the ERC20 token that approves the transfer
 * @param amount The amount of the Token to be approved for transfer
 * @param signer The signer that approves the transfer on his behalf
 */
export async function approveTokenTransfer({ contractAddress, tokenAddress, amount, signer }) {
    const tokenContract = new Contract(tokenAddress, ERC20_ABI, signer);
    if (typeof amount !== 'string') {
        if (BigNumber.isBigNumber(amount)) {
            amount = amount.toString();
        }
        else {
            amount = amount.toString(10);
        }
    }
    const tx = await tokenContract['approve'](contractAddress, amount);
    return await tx.wait();
}
/**
 * Fetches the transfer allowance of a contract for a signer on an ERC20 token
 * @param {string} contractAddress The address that wants to transfer tokens on behalf of a signer
 * @param {string} tokenAddress The address of the ERC20 token
 * @param {string} signerAddress The address of the signer that owns the tokens
 * @param {Provider} provider An RPC provider to make the request
 * @returns {Promise<bigint>} The transfer allowance of the contract
 */
export async function getAllowance(contractAddress, tokenAddress, signerAddress, provider) {
    const tokenContract = new Contract(tokenAddress, ERC20_ABI, provider);
    const allowance = await tokenContract['allowance'](signerAddress, contractAddress);
    return BigInt(allowance.toString(10));
}
//# sourceMappingURL=approveTokenTransfer.js.map